@@ -1,13 +1,20 @@
 """Data models."""
 
+from __future__ import annotations
+
 from typing import Literal
 
 from pydantic import BaseModel
 
 
-class _ImportedFrom(BaseModel):
+class _ImportedFromProject(BaseModel):
     project: str
-    path: str = None
+    path: str | None = None
+    git_rev: str | None = None
+
+
+class _ImportedFromUrl(BaseModel):
+    url: str
 
 
 class _CalkitObject(BaseModel):
@@ -21,6 +28,10 @@ class Dataset(_CalkitObject):
     pass
 
 
+class ImportedDataset(Dataset):
+    imported_from: _ImportedFromProject | _ImportedFromUrl
+
+
 class Figure(_CalkitObject):
     pass
 
@@ -49,9 +60,18 @@ class ReferenceCollection(BaseModel):
 
 
 class Environment(BaseModel):
-    name: str
-    kind: Literal["conda", "docker", "pip", "poetry", "npm", "yarn"]
-    path: str
+    kind: Literal[
+        "conda", "docker", "pip", "poetry", "npm", "yarn", "remote-ssh"
+    ]
+    path: str | None = None
+    description: str | None = None
+    stage: str | None = None
+
+
+class DockerEnvironment(Environment):
+    kind: str = "docker"
+    image: str
+    layers: list[str] | None = None
 
 
 class Software(BaseModel):
@@ -76,6 +96,6 @@ class ProjectInfo(BaseModel):
     figures: list[Figure] = []
     publications: list[Publication] = []
     references: list[ReferenceCollection] = []
-    environments: list[Environment] = []
+    environments: dict[str, Environment | DockerEnvironment] = {}
     software: list[Software] = []
     notebooks: list[Notebook] = []
